# gozerbot/config.py
#
#

""" this is where the config dict lives .. use pickle to persist config data
    .. use this pickle on start until the config file has changed  """

__copyright__ = 'this file is in the public domain'


from gozerbot.datadir import datadir
import os, pickle, subprocess, sys, traceback, types, thread

# version string
ver = 'GOZERBOT 0.8.2-BETA2'

def handle_exception():
    exctype, excvalue, tb = sys.exc_info()
    trace = traceback.extract_tb(tb)
    result = ""
    for i in trace:  
        fname = i[0]
        linenr = i[1]
        func = i[2]
        result += "%s:%s %s | " % (fname, linenr, func)
    del trace
    print "%s%s: %s" % (result, exctype, excvalue)

def diffdict(dictfrom, dictto):
    """ check for differences between two dicts """
    temp = {}
    for i in dictto.iteritems():
        if dictfrom.has_key(i[0]):
            if dictfrom[i[0]] != i[1]:
                temp.setdefault(i[0], i[1])
        else:
            temp.setdefault(i[0], i[1])
    return temp

class Config(dict):

    """ config object is a dict """

    def __init__(self, ddir, *args, **kw):
        dict.__init__(self, *args, **kw)
        self.dir = str(ddir)
        self.cfile = self.dir + os.sep + 'config'
        self.configlist = []
        self.lock = thread.allocate_lock()
        self['dbtype'] = 'mysql'

    def __getitem__(self, item):
        """ get config item .. return None if not available"""
        if not self.has_key(item):
            return None
        else:
            return dict.__getitem__(self, item)

    def set(self, item, value):
        """ set a config item """
        dict.__setitem__(self, item, value)
        self.save()

    def load(self):
        """ load the config file """
        # first do reload of the data/config file
        self.reload()
        # set version
        if self['dbenable']:
            self['version'] = ver + ' ' + self['dbtype'].upper()
        else:
            self['version'] = ver + ' ' + 'PICKLE'
    
    def save(self):
        try:
            self.lock.acquire()
            self.configlist = open(self.cfile, 'r').readlines()
            configtmp = open(self.cfile + '.tmp', 'w')
            teller = 0
            for line in self.configlist:
                teller += 1
                if line.startswith('#'):
                    configtmp.write(line)
                    continue
                try:
                    keyword = line.split()[0]
                except IndexError:
                    configtmp.write(line)
                    continue
                if self.has_key(keyword):
                    if type(self[keyword]) == types.StringType:
                        configtmp.write("%s = '%s'\n" % (keyword, self[keyword]))
                    elif type(self[keyword]) == types.UnicodeType:
                        configtmp.write("%s = u'%s'\n" % (keyword, self[keyword]))
                    else:
                        configtmp.write("%s = %s\n" % (keyword, str(self[keyword])))
                else:
                    configtmp.write(line)
            configtmp.close()
            os.rename(self.cfile + '.tmp', self.cfile)
            self.lock.release()
            return teller
        except Exception, ex:
            print "ERROR WRITING CONFIG FILE: %s" % str(ex)
            try:
                self.lock.release()
            except:
                pass
            return

    def reload(self):
        """ use execfile to reload data/config """
        try:
            execfile(self.dir + os.sep + 'config', config)
        except IOError:
            self.defaultconfig()
        except Exception, ex:
            print "ERROR READING CONFIG FILE: %s" % str(ex)
        # remove builtin data
        del self['__builtins__']
        # set version
        if self['dbenable']:
            self['version'] = ver + ' ' + self['dbtype'].upper()
        else:
            self['version'] = ver + ' ' + 'PIKCLE'

    def defaultconfig(self):
        """ init default config values if no config file is found """
        self['sqlite3'] = 0
        self['loglevel'] = 100
        self['jabberenable'] = 0
        self['ircdisable'] = 0
        self['stripident'] = 1
        self['owneruserhost'] = ['bart@127.0.0.1', ]
        self['nick'] = 'gozerbot'
        self['server'] = 'localhost'
        self['port'] = 6667
        self['ipv6'] = 0
        self['ssl'] = 0
        self['username'] = 'gozerbot'
        self['realname'] = 'GOZERBOT'
        self['defaultcc'] = "!"
        self['nolimiter'] = 0
        self['quitmsg'] = 'http://gozerbot.org'
        self['dbenable'] = 0
        self['dbtype'] = 'sqlite'
        self['udp'] = 0
        self['partyudp'] = 0
        self['mainbotname'] = 'main'
        self['addonallow'] = 0
        self['allowedchars'] = []

configtxt = """# config
#
#

__copyright__ = 'this file is in the public domain'

# use sqlite3 as default .. disabled this if you want other then sqlite3 support
sqlite3 = 1

# gozerdata dir umask
umask = 0700

# logging level .. the higher this value is the LESS the bot logs
loglevel = 10

## jabber section:

jabberenable = 0
jabberowner = 'bartholo@localhost'
jabberhost = 'localhost'
jabberuser = 'gozerbot@localhost'
jabberpass = 'pass'
jabberoutsleep = 0.1

## irc section:

ircdisable = 0

# stripident .. enable stripping of ident from userhost
stripident = 1

# userhost of owner .. make sure this matches your client's userhost
# if it doesn't match you will get an userhost denied message when you
# try to send commands to the bot
owneruserhost = ['dunker@127.0.0.1', ]

# the nick the bot tries to use, only used if no nick is set
# otherwise the bot will use the last nick used by the !nick command
nick = 'gozerbot'

# alternick
#alternick = 'gozerbot2'

# server to connect to
server = 'localhost'

# irc port to connect to 
port = 6667

# ircd password for main bot
#password = 'bla'

# ipv6
ipv6 = 0

# ssl
ssl = 0

# bindhost .. uncomment and edit to use
#bindhost = 'localhost'

# bots username
username = 'gozerbot'

# realname
realname = 'GOZERBOT'

# default control character
defaultcc = "!"

# no limiter
nolimiter = 0

# quit message
quitmsg = 'http://gozerbot.org'

# nickserv .. set pass to enable nickserv ident
nickservpass = ""
nickservtxt = ['set unfiltered on', ]

## if you want to use a database:

dbenable = 1 # set to 1 to enable
dbtype = 'sqlite' # one of mysql or sqlite
dbname = "db/users.db"
dbhost = "localhost"
dbuser = "bart"
dbpasswd = "mekker2"
dboldstyle = False # set to True if mysql database is <= 4.1
dboldsqlite = False

## if you want to use udp:

# udp
udp = 0 # set to 1 to enable
partyudp = 0
udpipv6 = 0
udphost = 'localhost'
udpport = 5500
udpmasks = ['192.168*', ]
udpallow = ['127.0.0.1', ]
udpallowednicks = ['#dunkbots', 'dunker']
udppassword = 'mekker'
udpseed = "" # set this to 16 char wide string if you want to encrypt the data
udpstrip = 1 # strip all chars < char(32)
udpsleep = 0 # sleep in sendloop .. can be used to delay packet traffic

# tcp
tcp = 0 # set to 1 to enable
partytcp = 0
tcpipv6 = 0
tcphost = 'localhost'
tcpport = 5500 
tcpmasks = ['192.168*', ]
tcpallow = ['127.0.0.1', ]
tcpallowednicks = ['#dunkbots', 'dunker', 'dunker@jabber.xs4all.nl']
tcppassword = 'mekker'
tcpseed = "bla1234567890bla"
# set this to 16 char wide string if you want to encrypt the data
tcpstrip = 1
tcpsleep = 0   

## other stuff:

# plugin server 
pluginserver = 'http://gozerbot.org'

# upgradeurl .. only needed if mercurial repo changed
#upgradeurl = 'http://core.gozerbot.org/hg/gozerbot'

# mail related
mailserver = None
mailfrom = None

# collective boot server
collboot = "gozerbot.org:8088"

# name of the main bot
mainbotname = 'main'

# allowed character for strippedtxt
allowedchars = []

# set to 1 to allow addons
addonallow = 0

# enable loadlist
loadlist = 0

# allow every command except OPER commands
auto_register = 0
"""

def writeconfig():
    """ wtite default config file to datadir/config """
    if not os.path.isfile(datadir + os.sep + 'config'):
        cfgfile = open(datadir + os.sep + 'config', 'w')
        cfgfile.write(configtxt)
        cfgfile.close()

loadlist = """
core
misc
irc
not
grep
reverse
count
chanperm
choice
fleet
ignore
upgrade
job
reload
rest
tail
user
googletalk
all
at
backup
install
reload
tell
reverse
to
underauth
userstate
alias
nickserv
"""

def writeloadlist():
    """ write loadlist to datadir """
    if not os.path.isfile(datadir + os.sep + 'loadlist'):
        cfgfile = open(datadir + os.sep + 'loadlist', 'w')
        cfgfile.write(loadlist)
        cfgfile.close()

# create the config dict and load it from file
config = Config(datadir)
