# gozerbot/runner.py
#
#

""" threads management to run jobs """

__copyright__ = 'this file is in the public domain'

from gozerbot.thr import start_new_thread
from gozerbot.generic import handle_exception, rlog, lockdec
from gozerbot.threadloop import ThreadLoop
from gozerbot.periodical import minutely
import Queue, time, thread, random

runlock = thread.allocate_lock()
locked = lockdec(runlock)

class Runner(ThreadLoop):

    def __init__(self, name=""):
        ThreadLoop.__init__(self, name)
        self.working = False
        self.starttime = time.time()
        self.elapsed = self.starttime
        self.finished = time.time()

    def handle(self, descr, func, *args, **kwargs):
        self.working = True
        self.name = descr
        try:
            rlog(0, 'runner', 'running %s: %s' % (descr, str(func)))
            self.starttime = time.time()
            func(*args, **kwargs)
            self.finished = time.time()
            self.elapsed = self.finished - self.starttime
            if self.elapsed > 3:
                rlog(10, 'runner', 'ALERT %s %s job taking too long: %s \
seconds' % (descr, str(func), self.elapsed))
        except Exception, ex:
            handle_exception()
        self.working = False

class CommandRunner(Runner):

    def handle(self, descr, func, bot, ievent, *args, **kwargs):
        self.working = True
        try:
            self.name = descr
            rlog(10, 'runner', '%s (%s) running %s: %s at speed %s' % \
(ievent.nick, ievent.userhost, descr, str(func), ievent.speed))
            self.starttime = time.time()
            func(bot, ievent, *args, **kwargs)
            for queue in ievent.queues:
                queue.put_nowait(None)
            self.finished = time.time()
            self.elapsed = self.finished - self.starttime
            if self.elapsed > 3:
                rlog(10, 'runner', 'ALERT %s %s job taking too long: %s \
seconds' % (descr, str(func), self.elapsed))
        except Exception, ex:
            handle_exception(ievent)
        self.working = False

class Runners(object):

    def __init__(self, max=100, runnertype=Runner):
        self.max = max
        self.runners = []
        self.cleanup()
        self.runnertype=runnertype

    def runnersizes(self):
        result = []
        for runner in self.runners:
            result.append(runner.queue.qsize())
        return result

    def stop(self):
        for runner in self.runners:
            runner.stop()
        self.cleanup()

    def start(self):
        pass
 
    def put(self, *data):
        launched = False
        for runner in self.runners:
            if not runner.working and runner.running:
                runner.put(*data)
                launched = True
                return
        runner = self.makenew()
        runner.put(*data)

    def running(self):
        result = []
        for runner in self.runners:
            if runner.working:
                result.append(runner.nowrunning)
        return result

    def makenew(self):
        runner = None
        if len(self.runners) < self.max:
            runner = self.runnertype()
            runner.start()
            self.runners.append(runner)
        else:
            runner = random.choice(self.runners)
        return runner

    @minutely
    def cleanup(self):
        rlog(-10, 'runners', 'running cleanup')
        for runner in self.runners:
            if not runner.working:
                runner.stop() 
                self.runners.remove(runner)

cbrunners = [Runners(10-i) for i in range(10)]
cmndrunners = [Runners(12-i, CommandRunner) for i in range(10)]

def runners_start():
    for runner in cbrunners:
        runner.start()
    for runner in cmndrunners:
        runner.start()

def runners_stop():
    for runner in cbrunners:
        runner.stop()
    for runner in cmndrunners:
        runner.stop()
